unit KADaoDBGrid;
{$I CommonDirectives.pas}
interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, Grids, DBGrids, Menus;


type
  TKADaoDBGrid = class(TDBGrid)
  private
    { Private declarations }
    F_TopRect      : TRect;
    F_Clicked      : Boolean;
    F_PopupMenu    : TPopupMenu;
    F_MenuBMP      : TBitmap;
    F_MenuTitles   : TStringList;
    F_LastFilter   : TStringList;
    F_FrameIndex   : Boolean;
    F_ShowGridMenu : Boolean;
    procedure        F_Set_MenuTitles(Value:TStringList);
    procedure        F_Set_ShowGridMenu(Value : Boolean);
    procedure        F_Set_FrameIndex(Value : Boolean);
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState); override;
    procedure DrawColumnCell(const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState); override;
    procedure Loaded; override;
    Procedure OnSelectIndex(Sender: TObject);
    Procedure OnSortBy(Sender: TObject);
    Procedure OnFilterBy(Sender: TObject);
    Procedure OnFind(Sender: TObject);
    Procedure OnSeek(Sender: TObject);
    Procedure OnQuickFind(Sender: TObject);
    Procedure OnQuickSeek(Sender: TObject);
    Procedure OnShowMemos(Sender: TObject);
    Procedure OnFastLookup(Sender: TObject);
    Procedure OnFrameIndexFields(Sender: TObject);
    Procedure OnChooseVisibleFields(Sender: TObject);
    Procedure OnSaveAs(Sender: TObject);
  public
    { Public declarations }
    Property    GridPopupMenu : TPopupMenu  Read  F_PopupMenu;
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
  published
    { Published declarations }
    Property ShowGridMenu     : Boolean     Read F_ShowGridMenu Write F_Set_ShowGridMenu;
    Property MenuTitles       : TStringList Read F_MenuTitles   Write F_Set_MenuTitles;
    Property FrameIndexFields : Boolean     Read F_FrameIndex   Write F_Set_FrameIndex;
  end;

procedure Register;

implementation
{$R KADaoDBGrid.res}
Uses Forms, Dialogs, DB, KDaoTable, DaoApi, KADaoSelectIndexUnit,
  KADaoFilterByUnit, KADaoSortByDialogUnit, KADaoFindSeekUnit,
  KADaoQuickFindSeekUnit, KADaoCVFUnit, KADaoSASUnit
  {$IFDEF D6UP}, Variants{$ENDIF};

Const
  NumMenus = 16;

constructor TKADaoDBGrid.Create(AOwner: TComponent);
Var
 MI : TMenuItem;
 X  : Integer;
Begin
 Inherited Create(AOwner);
 F_ShowGridMenu    := True;
 F_Clicked         := False;
 F_MenuBMP         := TBitmap.Create;
 F_MenuBMP.LoadFromResourceName(HInstance, 'MENUBMP');
 F_MenuTitles      := TStringList.Create;
 F_LastFilter      := TStringList.Create;
 F_FrameIndex      := False;

 //*************************************************************
 F_PopupMenu       := TPopupMenu.Create(Self);
 F_PopupMenu.Name  := 'OptionsMenu';
 For X := 1 to NumMenus do
     Begin
      MI := TMenuItem.Create(F_PopupMenu);
      F_PopupMenu.Items.Insert(0,MI);
     End;

 {00} F_MenuTitles.Add('Select Index');
 {01} F_MenuTitles.Add('Sort');
 {02} F_MenuTitles.Add('Filter');
 {03} F_MenuTitles.Add('-');
 {04} F_MenuTitles.Add('Find');
 {05} F_MenuTitles.Add('Seek');
 {06} F_MenuTitles.Add('Quick Find');
 {07} F_MenuTitles.Add('Quick Seek');
 {08} F_MenuTitles.Add('-');
 {09} F_MenuTitles.Add('Show Memos');
 {10} F_MenuTitles.Add('Fast Lookup');
 {11} F_MenuTitles.Add('Frame Index Fields');
 {12} F_MenuTitles.Add('-');
 {13} F_MenuTitles.Add('Choose Visible Fields');
 {14} F_MenuTitles.Add('-');
 {15} F_MenuTitles.Add('Save as');


 F_PopupMenu.Items[0].OnClick  := OnSelectIndex;
 F_PopupMenu.Items[1].OnClick  := OnSortBy;
 F_PopupMenu.Items[2].OnClick  := OnFilterBy;
 F_PopupMenu.Items[4].OnClick  := OnFind;
 F_PopupMenu.Items[5].OnClick  := OnSeek;
 F_PopupMenu.Items[6].OnClick  := OnQuickFind;
 F_PopupMenu.Items[7].OnClick  := OnQuickSeek;
 F_PopupMenu.Items[9].OnClick  := OnShowMemos;
 F_PopupMenu.Items[10].OnClick := OnFastLookup;
 F_PopupMenu.Items[11].OnClick := OnFrameIndexFields;
 F_PopupMenu.Items[13].OnClick := OnChooseVisibleFields;
 F_PopupMenu.Items[15].OnClick := OnSaveAs;
 //*************************************************************
End;

destructor  TKADaoDBGrid.Destroy;
Begin
 F_MenuBMP.Free;
 F_PopupMenu.Free;
 F_MenuTitles.Free;
 F_LastFilter.Free;
 Inherited Destroy;
End;

procedure TKADaoDBGrid.Loaded;
Var
 X : Integer;
 S : String;
Begin
  Inherited Loaded;
  For X := 0 To F_MenuTitles.Count-1 do
      Begin
       if X < F_PopupMenu.Items.Count Then
          Begin
            S:=F_MenuTitles.Strings[X];
            if S[1]='!' Then System.Delete(S,1,1);
            F_PopupMenu.Items[X].Caption:=S;
          End;
      End;
End;

procedure TKADaoDBGrid.F_Set_MenuTitles(Value:TStringList);
Begin
 F_MenuTitles.Assign(Value);
End;

procedure TKADaoDBGrid.F_Set_ShowGridMenu(Value : Boolean);
Begin
 F_ShowGridMenu:=Value;
 Self.Refresh;
End;

procedure TKADaoDBGrid.F_Set_FrameIndex(Value : Boolean);
Begin
 F_FrameIndex := Value;
 F_PopupMenu.Items[11].Checked := Value;
 Self.Refresh;
End;

procedure TKADaoDBGrid.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Cell      : TGridCoord;
  SaveState : TGridState;
  Rect      : TRect;
Begin
   SaveState := FGridState;
   if (SaveState = gsRowSizing) or (SaveState = gsColSizing) or (Not F_ShowGridMenu) or
      ((InplaceEditor <> nil) and (InplaceEditor.Visible) and
      (PtInRect(InplaceEditor.BoundsRect, Point(X,Y)))) then
       Begin
         inherited MouseDown(Button, Shift, X, Y);
         Exit;
       End;
 Cell := MouseCoord(X,Y);
 if (Button = mbLeft) and (Cell.X =0 ) and (Cell.Y = 0) then
     Begin
       //***********************************************************************
       // Upper Left Rectange is clicked - draw a flat style fixed cell
       //***********************************************************************
       F_Clicked := True;
       Rect := F_TopRect;
       Canvas.Brush.Color := FixedColor;
       Canvas.FillRect(Rect);
       InflateRect(Rect, 1, 1);
       DrawEdge(Canvas.Handle, Rect, BDR_RAISEDINNER, BF_FLAT);
       DrawEdge(Canvas.Handle, Rect, BDR_RAISEDINNER, BF_FLAT);
       Canvas.Draw(((Rect.Left+Rect.Right-F_MenuBMP.Width) div 2)+1,((Rect.Top + Rect.Bottom - F_MenuBMP.Height) div 2)+1, F_MenuBMP);
     End;
 inherited MouseDown(Button, Shift, X, Y);
End;

procedure TKADaoDBGrid.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Cell      : TGridCoord;
  SaveState : TGridState;
  Rect      : TRect;
  ShowMenu  : Boolean;
  MPT       : TPoint;
begin
  ShowMenu  := False;
  SaveState := FGridState;
  if (SaveState = gsRowSizing) or (SaveState = gsColSizing) or (Not F_ShowGridMenu) or
     ((InplaceEditor <> nil) and (InplaceEditor.Visible) and
     (PtInRect(InplaceEditor.BoundsRect, Point(X,Y)))) then
     Begin
       inherited MouseUp(Button, Shift, X, Y);
       Exit;
     End;
  Cell := MouseCoord(X,Y);
  if ((Button = mbLeft) and (Cell.X =0 ) and (Cell.Y = 0)) Or (F_Clicked) then
     Begin
       //***********************************************************************
       // Upper Left Rectange is clicked -
       // draw a raised style fixed cell and popup a context menu
       //***********************************************************************
        Rect := F_TopRect;
        Canvas.Brush.Color := FixedColor;
        Canvas.FillRect(Rect);
        DrawEdge(Canvas.Handle, Rect, BDR_RAISEDINNER, BF_BOTTOMRIGHT);
        DrawEdge(Canvas.Handle, Rect, BDR_RAISEDINNER, BF_TOPLEFT);
        F_Clicked := False;
        ShowMenu  := True;
       //***********************************************************************
       Button:=mbRight;
     End;
  inherited MouseUp(Button, Shift, X, Y);
  if (ShowMenu)  Then
     Begin
       MPT.X := F_TopRect.Left;
       MPT.Y := F_TopRect.Bottom;
       MPT := ClientToScreen(MPT);
       DrawCell(0,0,F_TopRect,[gdFixed]);
       if  Assigned(DataSource)
       And Assigned(DataSource.Dataset)
       And (DataSource.DataSet is TKaDaoTable)
       And (DataSource.DataSet.Active) Then
           Begin
             For X := 0 to NumMenus-1 do F_PopupMenu.Items[X].Enabled := True;
             if (DataSource.DataSet as TKaDaoTable).TableType=dbOpenTable Then
                Begin
                  F_PopupMenu.Items[1].Enabled := False;
                  F_PopupMenu.Items[2].Enabled := False;
                  F_PopupMenu.Items[4].Enabled := False;
                  F_PopupMenu.Items[6].Enabled := False;
                  if (DataSource.DataSet as TKaDaoTable).IndexFieldCount=0 Then
                     Begin
                       F_PopupMenu.Items[5].Enabled := False;
                       F_PopupMenu.Items[7].Enabled := False;
                       F_PopupMenu.Items[11].Enabled := False;
                     End;
                End
             Else
                Begin
                  F_PopupMenu.Items[0].Enabled  := False;
                  F_PopupMenu.Items[5].Enabled  := False;
                  F_PopupMenu.Items[7].Enabled  := False;
                  F_PopupMenu.Items[11].Enabled := False;
                End;
             For X := 0 to NumMenus-1 do
                 Begin
                   if F_MenuTitles.Strings[X][1]='!' Then F_PopupMenu.Items[X].Enabled := False;
                 End;
             If (DataSource.DataSet.IsEmpty) Then F_PopupMenu.Items[15].Enabled := False;
             F_PopupMenu.Popup(MPT.X,MPT.Y);
           End;
     End;
End;

procedure TKADaoDBGrid.DrawCell(ACol, ARow: Longint; ARect: TRect; AState: TGridDrawState);
Begin
 inherited DrawCell(ACol, ARow, ARect, AState);
 if AState<>[gdFixed] Then Exit;
 if Not F_ShowGridMenu Then Exit;
 if (ACol=0) And (ARow=0) Then
    Begin
      F_TopRect := ARect;
      Canvas.Draw((ARect.Left+ARect.Right-F_MenuBMP.Width) div 2,(ARect.Top + ARect.Bottom - F_MenuBMP.Height) div 2, F_MenuBMP);
    End
 Else
    Begin
    End;
End;

procedure TKADaoDBGrid.DrawColumnCell(const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
Var
  X : Integer;
Begin
 Inherited DrawColumnCell(Rect,DataCol,Column,State);
 if Not F_FrameIndex Then Exit;
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       if (DataSource.DataSet as TKaDaoTable).IndexFieldCount=0 Then Exit;
       For X := 0 To  (DataSource.DataSet as TKaDaoTable).IndexFieldCount-1 do
           Begin
             if AnsiLowerCase(Column.Field.FieldName)= AnsiLowerCase((DataSource.DataSet as TKaDaoTable).IndexFields[X].FieldName) Then
                Begin
                  Canvas.Brush.Color:=clRed;
                  Canvas.FrameRect(Rect);
                End;
           End;
     End;
End;

//******************************************************************************
Procedure TKADaoDBGrid.OnSelectIndex(Sender: TObject);
Var
  NewIndex : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
    Begin
      if (DataSource.DataSet as TKaDaoTable).TableType=dbOpenTable Then
         Begin
           NewIndex:=(DataSource.DataSet as TKaDaoTable).IndexName;
           Application.CreateForm(TKADaoSelectIndex, KADaoSelectIndex);
           if KADaoSelectIndex.Execute((DataSource.DataSet as TKaDaoTable), NewIndex) Then
              Begin
              End;
           KADaoSelectIndex.Free;
           (DataSource.DataSet as TKaDaoTable).IndexName := NewIndex;
         End;
    End;
End;

Procedure TKADaoDBGrid.OnSortBy(Sender: TObject);
Var
 DT : TKADaoTable;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
      DT := (DataSource.DataSet as TKaDaoTable);
      Application.CreateForm(TKADaoSortByDialog,KADaoSortByDialog);
      if KADaoSortByDialog.Execute(DT.FieldNames,DT.SortedBy,DT.UseBrackets) Then
       Begin
         DT.Sort;
       End;
      KADaoSortByDialog.Free;
     End;
End;

Procedure TKADaoDBGrid.OnFilterBy(Sender: TObject);
Var
  NewFilter : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
      Application.CreateForm(TKADaoFilterBy, KADaoFilterBy);
      NewFilter:=(DataSource.DataSet as TKaDaoTable).Filter;
      if KADaoFilterBy.Execute((DataSource.DataSet as TKaDaoTable), NewFilter, F_LastFilter) Then
         Begin
           if NewFilter  <> '' Then
              Begin
               (DataSource.DataSet as TKaDaoTable).Filter   := NewFilter;
               (DataSource.DataSet as TKaDaoTable).Filtered := True;
              End
           Else
              Begin
               (DataSource.DataSet as TKaDaoTable).Filtered := False;
              End;
         End;
      KADaoFilterBy.Free;
     End;
End;

Procedure TKADaoDBGrid.OnFind(Sender: TObject);
Var
 BK : TBookmarkStr;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
          BK := (DataSource.DataSet as TKaDaoTable).Bookmark;
       Application.CreateForm(TKADaoFindSeekDialog, KADaoFindSeekDialog);
       if KADaoFindSeekDialog.Execute(DataSource.DataSet as TKaDaoTable, False) Then
          Begin
          End
       Else
          Begin
           if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
              (DataSource.DataSet as TKaDaoTable).Bookmark:=BK;
          End;
       KADaoFindSeekDialog.Free;
     End;
End;

Procedure TKADaoDBGrid.OnSeek(Sender: TObject);
Var
 BK : TBookmarkStr;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active)
 And ((DataSource.DataSet As TKaDaoTable).IndexFieldCount > 0)  Then
     Begin
       if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
          BK := (DataSource.DataSet as TKaDaoTable).Bookmark;
       Application.CreateForm(TKADaoFindSeekDialog, KADaoFindSeekDialog);
       if KADaoFindSeekDialog.Execute(DataSource.DataSet as TKaDaoTable, True) Then
          Begin
          End
       Else
          Begin
           if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
              (DataSource.DataSet as TKaDaoTable).Bookmark:=BK;
          End;
       KADaoFindSeekDialog.Free;
     End;
End;

Procedure TKADaoDBGrid.OnQuickFind(Sender: TObject);
Var
 BK : TBookmarkStr;
 FN : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       FN := Self.SelectedField.FieldName;
       if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
          BK := (DataSource.DataSet as TKaDaoTable).Bookmark;
       Application.CreateForm(TQFS, QFS);
       if QFS.Execute(DataSource.DataSet as TKaDaoTable, False, FN) Then
          Begin
          End
       Else
          Begin
           if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
              (DataSource.DataSet as TKaDaoTable).Bookmark:=BK;
          End;
       QFS.Free;
     End;
End;

Procedure TKADaoDBGrid.OnQuickSeek(Sender: TObject);
Var
 BK : TBookmarkStr;
 FN : String;
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active)
 And ((DataSource.DataSet As TKaDaoTable).IndexFieldCount > 0)  Then
     Begin
       FN := Self.SelectedField.FieldName;
       if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
          BK := (DataSource.DataSet as TKaDaoTable).Bookmark;
       Application.CreateForm(TQFS, QFS);
       if QFS.Execute(DataSource.DataSet as TKaDaoTable, True, FN) Then
          Begin
          End
       Else
          Begin
           if (DataSource.DataSet as TKaDaoTable).Bookmarkable Then
              (DataSource.DataSet as TKaDaoTable).Bookmark:=BK;
          End;
       QFS.Free;
     End;
End;

Procedure TKADaoDBGrid.OnShowMemos(Sender: TObject);
Begin
 F_PopupMenu.Items[9].Checked := Not F_PopupMenu.Items[9].Checked;
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     (DataSource.DataSet as TKaDaoTable).CacheMemos := F_PopupMenu.Items[9].Checked;
End;

Procedure TKADaoDBGrid.OnFastLookup(Sender: TObject);
Begin
 F_PopupMenu.Items[10].Checked := Not F_PopupMenu.Items[10].Checked;
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     (DataSource.DataSet as TKaDaoTable).CacheLookups := F_PopupMenu.Items[10].Checked;
End;

Procedure TKADaoDBGrid.OnFrameIndexFields(Sender: TObject);
Begin
 F_PopupMenu.Items[11].Checked := Not F_PopupMenu.Items[11].Checked;
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       F_FrameIndex := F_PopupMenu.Items[11].Checked;
       Self.Refresh;
     End;
End;

Procedure TKADaoDBGrid.OnChooseVisibleFields(Sender: TObject);
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (DataSource.DataSet.Active) Then
     Begin
       Application.CreateForm(TCVF, CVF);
       if CVF.Execute(DataSource.DataSet) Then
          Begin
          End;
       CVF.Free;
       Self.Refresh;
     End;
End;

Procedure TKADaoDBGrid.OnSaveAs(Sender: TObject);
Begin
 if  Assigned(DataSource)
 And Assigned(DataSource.Dataset)
 And (DataSource.DataSet is TKaDaoTable)
 And (NOT DataSource.DataSet.IsEmpty)
 And (DataSource.DataSet.Active) Then
     Begin
       Application.CreateForm(TSAS, SAS);
       if SAS.Execute(DataSource.DataSet) Then
          Begin
          End;
       SAS.Free;
     End;
End;

procedure Register;
begin
  RegisterComponents('KADao Controls', [TKADaoDBGrid]);
end;

end.
